default(recover,0);
default(strictargs,1);
default(parisizemax,1500*10^6);
read("../vecper.gp");

\\------------------------------------------------------------------------------
\\ Even Period

print("even");
{
  if(0,
    my(limit=1000);
    for(i=1,100,
       my(num=random(limit),
          den=random(limit-2)+2);
       num %= den;
       my(fra=num/den,
          v=vecper_frac_digits(num/den),
          len=vecper_periodic_length(v));
       len%2==0 || next;
       my(gnum=Polrev(vecper_periodic(v)),
          gden=1-x^len,
          halflen=len/2,
          e=(1-x^halflen)/(1-x),
          both=gnum%e==0 && gden%e==0);
       if(both,next);

       print("fra = ",fra,"  ",v,"  ",len%2);
       print("  e   = ",e);
       print("  gcd = ",gcd(gnum,gden));
       print("  ",both);
    );
    quit);
}


\\------------------------------------------------------------------------------


OEIS_samples("A230979")

\\ fixed     0
\\ periodic  880014
\\      den 2053373

3*880014
2053373

\\ factor(2053373-1)
\\ factor(2053373)
2053373 == 7 * 293339         || error();
eulerphi(2053373) == 1760028  || error();
eulerphi(2053373) == 1760028  || error();
1760028/2 == 880014           || error();

num = 9823
den = 2053373;
periodic_length=880014

eulerphi(2053373)/2

\\ num=2;den=7;periodic_length=12

fra = num/den;
half_length = periodic_length/2;
print("fra ",fra);
my(n=num,h); digs = vector(periodic_length,i, [h,n]=divrem(10*n,den); h);
print("digs len ",#digs);
v=[[],digs];  \\ vecper
print("normalize ...");
v == vecper_normalize(v)  || error();

{
  if(1,
     print("vecper_frac_fromdigits ... (slow-ish)");
     fra_back = vecper_frac_fromdigits(v);
     print("fraction back ",fra_back);
     fra_back == fra || error();
    );
}

print("digs length ",#digs);
digs1 = digs[1..periodic_length/2];
digs2 = digs[periodic_length/2+1..#digs];
#digs1 == #digs2 || error();
print("eq halves ",digs1==digs2);

g_num = Polrev(digs);
g_den = 1-x^periodic_length;
print("g_num degree ",poldegree(g_num));
print("g_den degree ",poldegree(g_den));

g_den_minus = 1-x^(periodic_length/2);
g_den_plus  = 1+x^(periodic_length/2);
g_den_minus*g_den_plus == g_den  || error();

{
  print("remainders");
  print(g_num % (1+x));
  print(g_num % (1-x));
  if(0,
     print(g_num % g_den_minus == 0);
     print(g_num % g_den_plus == 0));
}

hh = (1-x^half_length)/(1-x);
print("hh degree "poldegree(hh));
g_den % hh == 0
g_num % hh == 0

g_num2=Mod(g_num,2);
g_den2=Mod(g_den,2);
print(g_num2 + O(x^15));
print("gcd2 ...");
gcd2 = gcd(g_num2,g_den2);
gcd2_deg = poldegree(gcd2);
print("  gcd2 degree ",gcd2_deg," away from half ",periodic_length/2 - gcd2_deg);
\\ gcd2
gcd2 == hh
print();

print("gcd ...");
g_gcd = gcd(g_num,g_den);
g_gcd_deg = poldegree(g_gcd);
print("  gcd degree ",g_gcd_deg);

print("divide ...");
g = g_num / g_den
print("g denominator degree ",poldegree(denominator(g)));

print("subst g ...");
subst(g*x,x,1/10) == fra  || error();
print("  good");

d_num = numerator(g);
d_den = denominator(g);
print("d_num "d_num"  over d_den "d_den);
q = g_den / d_den;
print("g_den / d_den =  ",q);
print("g_num / d_num =  ",g_num / d_num);
q == g_num / d_num



\\ forprime(m=2,9, print("g_num irreducible gf"m"  ",polisirreducible(Mod(g_num,m))));
\\ print("g_num irreducible ",polisirreducible(g_num));
\\ subst(g_num*x,x,1/10) / subst(g_den,x,1/10) == fra  || error();
\\ print("subst good");

quit


\\ read("vecper.gp");
\\ v=vecper_frac_digits(fra);
\\ vecper_fixed_length(v)
\\ vecper_periodic_length(v)
\\
quit




\\ quit
\\ 
\\ \\ gf2_gcd also divides full gnum,gden, not just down in GF(2) coeffs
\\ print("divisibility by gf2_gcd ... (gnum)");
\\ [gnum_q,gnum_r] = divrem(gnum, gf2_gcd);
\\ gnum_r == 0  || error();
\\ print("divisibility by gf2_gcd ... (gden)");
\\ [gden_q,gden_r] = divrem(gden, gf2_gcd);
\\ gden_r == 0  || error();
\\ 
\\ \\ This divisibility shows gf2_gcd is the full gcd of gnum,gden.
\\ \\ 
\\ \\ If gnum and gden have a common factor then it's also a common factor in
\\ \\ GF(2)[x], so has to be either gf2_gcd or some divisor of gf2_gcd, and in
\\ \\ this case it's the full gf2_gcd.  Hence linrec_order based on gf2_gcd.
\\ \\
\\ linrec_order == poldegree(gden_q)  || error();
\\ 
\\ print("fra by subst() of gnum / den (a bit slow) ...");
\\ subst(gnum,x,1/10) / subst(gden,x,1/10) == fra || error();
\\ 
\\ \\ All of the above is only actually a faster way to do generating function
\\ \\ g = gnum / gden.  But the common factor cancelling in the generic integer
\\ \\ coeffs seems rather slow at the 880014 length of gnum and gden.
\\ 
\\ print("generating function g = gnum/gden (a lot slow) ...");
\\ subst(g,x,1/10) == fra || error();

