\\ Copyright 2019, 2020, 2021, 2022, 2023 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
default(strictargs,1);
default(parisizemax,1500*10^6);
read("../vecper.gp");
read("../vecper-wip.gp");


\\-----------------------------------------------------------------------------
\\ Fibonacci Mod N

Fibonacci_mod_vecper(m) =
{
  vecper_make_state((state)->[lift(state[1]),
                              [state[2],vecsum(state)]],
                    [Mod(0,m), Mod(1,m)]);
}
v=Fibonacci_mod_vecper(5)
v=vecper_normalize(v)

print();
vector(20,n, vecper_periodic_length(Fibonacci_mod_vecper(n)))
\\ A001175 Pisano periods

vector(20,n, vecper_fixed_length(Fibonacci_mod_vecper(n)))
quit



\\-----------------------------------------------------------------------------
\\  1/n binary period 1

select(t->t==1,OEIS_data("A007733"),1)

{
  for(n=1,10,
     my(v=vecper_frac_digits(1/n,2));
        print(n"  "v));
   if(#v[2]==1,
    );
  quit;
}
     

\\-----------------------------------------------------------------------------
\\ A352023  1/prime without digit 9

find(v,elem) =
{
  for(i=1,vecsum(apply(length,v)),
     if(vecper_get(v,i)==elem, return(i)));
  'none;
}
lz(v) =
{
  for(i=1,oo,
     if(vecper_get(v,i)!=0, return(i-1)));
}
{
  for(p=10,200,
     my(v=vecper_frac_digits(1/p),
        pos=find(v,9));
     if(pos!='none,
        pos = pos - lz(v) - 1);
    print1(pos,", ");
     );

\\     print(pos"  of "vecper_periodic_length(v));
  quit;
}
\\     print1(pos-logint(p,10),", "));



\\-----------------------------------------------------------------------------
\\ A350814   largest digit of 1/n is exactly 3


\\ A114205  fixed part
\\ A060284  periodic part
\\ A266385  both fixed and periodic (up to first repetition)

\\ vector(30,n, fromdigits(vecper_fixed(vecper_frac_digits(1/n))))
\\ vector(30,n, fromdigits(vecper_periodic(vecper_frac_digits(1/n))))
 vector(20,n,n+=5; fromdigits(concat(vecper_frac_digits(1/n))))

vecper_frac_digits(1/2)

A266385_both(n) =
{
  if(n==1,1,
     my(v=vecper_frac_digits(1/n));
     fromdigits(if(v[2]==[0],v[1],concat(v))));
}
OEIS_check_func("A266385_both");


\\ not in OEIS: 16, 142857, 1250, 1, 10, 9, 83, 76923, 714285, 6, 6250, 588235294117647, 5, 52631578947368421, 50, 47619, 45, 434782608695652173913, 416, 40

is(n) = vecmax(concat(vecper_frac_digits(1/n)))==3;
is(n) = vecmax(digits(A266385_both(n)))==3;
want=[3, 30, 33, 75, 300, 303, 330, 333, 429, 750, 813, 3000, 3003, 3030, 3125, 3300, 3330, 3333, 4290, 4329, 7500, 7575, 8130, 30000, 30003, 30030, 30300, 30303, 31250, 33000, 33300, 33330, 33333, 42900, 43290, 46875, 75000, 75075, 75750, 76923, 81103, 81300, 300000];
{
  my(limit=2000);
  select(is,[1..limit]) == select(n->n<=limit,want)
}

\\ {
\\   for(n=1,20,
\\      my(v=vecper_frac_digits(1/n));
\\      v=vecper_fixed(v);
\\      print(n"  "v));
\\ }
quit

\\-----------------------------------------------------------------------------
\\ sqrt contfrac periodic parts

read("../../geom/geom.gp");

A121339 would be the periodic part, if like that for crossref or appear at suitable point.

my(n=9); vecper_contfrac(geom_sqrt_as_quad(n))
v=vector(39,n, vecper_periodic(vecper_contfrac(geom_sqrt_as_quad(n))))

select(n-> my(p=vecper_periodic(vecper_contfrac(geom_sqrt_as_quad(n)))); vecsort(p)==p,[1..147])
quit

want=[2, 3, 5, 6, 7, 8, 10, 11, 12, 13, 15, 17, 18, 20, 24, 26,\
               27, 30, 32, 35, 37, 38, 39, 40, 41, 42, 48, 50, 51, 55, 56,\
               58, 63, 65, 66, 68, 72, 74, 75, 80, 82, 83, 84, 87, 90, 99,\
               101, 102, 104, 105, 110, 120, 122, 123, 130, 132, 135, 136,\
               143, 145, 146, 147]
want == select(n-> my(p=vecper_periodic(vecper_contfrac(geom_sqrt_as_quad(n)))); #p>=1 && vecsort(p)==p,[1..147])
quit

v=concat(v)
#v
#OEIS_data("A121339")
v==OEIS_data("A121339")
\\ A121339 Periodic part of continued fraction for square roots of integers.
\\ A121339 ,2,1,2,4,2,4,1,1,1,4

contfrac(quadgen(20))

\\ v=select(n->!issquare(n), [1..100])
\\ c=apply(vecper_contfrac,v)

quit


\\-----------------------------------------------------------------------------
\\ Fibonacci mod 2^k
\\
\\ periodic length 3 6 12 24 48 96 192 384
\\ 3*2^(k-1)

Fibonacci_mod2k_vecper(k) =
{
  my(mask=bitneg(0,k));
  vecper_make_state((state)->state=[state[2],bitand(mask,state[1]+state[2])]; [state[1], state], [0,bitand(mask,1)]);
}
{
  \\ largest 8 n=30  832040 11001011001000101000
  \\ largest 10 n=38  39088169 10010101000111000000101001
  \\
  \\ plot(n=1,100,hammingweight(fibonacci(floor(n))))
  \\ vector(30,n,hammingweight(fibonacci(n)))  \\ A011373
  \\ vector(30,n,hammingweight(fibonacci(n))-hammingweight(fibonacci(n-1)))

  \\ A222295 Conjectured number of Fibonacci numbers with exactly n bits set in their binary representation.

  my(x=0,y=1,
     m=vector(30));
  for(n=1,100000,
     my(h=hammingweight(y));
     if(h<=#m, m[h]++);
     \\ if(hammingweight(x)==30,
     \\    print("n="n"  "x" "to_binary(x)));
     [x,y]=[y,x+y]);
  print(m);
  quit;
}

{
  my(k=8, vec=vecper_periodic(Fibonacci_mod2k_vecper(k)));
  for(i=1,#vec, printf("%*s\n", k, to_binary(vec[i])));
}

for(k=1,8,print(apply(to_binary,vecper_periodic(Fibonacci_mod2k_vecper(k)))));
for(k=1,8,print(Fibonacci_mod2k_vecper(k)));
for(k=1,8,print(vecper_periodic_length(Fibonacci_mod2k_vecper(k))));
quit




\\-----------------------------------------------------------------------------

vecper_frac_digits(1/10^4, 2)
v=vecper_frac_digits(93/2364125000000000);
vecper_fixed(v)
vecper_fixed_length(v)
vecper_periodic_length(v)
quit

vecper_frac_digits(1/10^4, 2)
v=vecper_frac_digits(93/2364125000000000);
vecper_fixed(v)
vecper_fixed_length(v)
vecper_periodic_length(v)
quit

\\ 0.014387768775 03933802146671602
\\ 18912
\\ 
f=272115870842319/18913000000000000
f*10^12 - 014387768775
f*10^12 - 014387768775.0
f*1.0
v=vecper_frac_digits(f);
vecper_fixed(v)
vecper_fixed_length(v)
vecper_periodic_length(v)

num=Pol(vecper_periodic(v));
den=(1-x^vecper_periodic_length(v));
print("gcd");
poldegree(gcd(num,den))
18912 - 9455 == 9457

g=Pol(vecper_periodic(v))/(1-x^vecper_periodic_length(v))
poldegree(denominator(g))
9457

quit

\\-----------------------------------------------------------------------------
read("../../geom/geom.gp");
vecper_contfrac(geom_sqrt_as_quad(13)/3)
[[1],           [4, 1, 20, 1, 4, 2]]
vecper_unroll(vecper_contfrac(geom_sqrt_as_quad(117)),3)
[[10], 1, 4, 2, [4, 1, 20, 1, 4, 2]]

[[10], [1, 4, 2, 4, 1, 20]]

\\ A010182 Continued fraction for sqrt(117).
sqrt(117)
10.81665382639196787935
sqrt(13)/3
 1.2018504251546630977
factor(117)
117==3^2 * 13
quit

\\-----------------------------------------------------------------------------
read("../../geom/geom.gp");
vector(30,n, vecper_periodic_length(vecper_contfrac(geom_sqrt_as_quad(n))))
\\ A003285 period of contrfrac
quit

\\-----------------------------------------------------------------------------

v=vecper_periodic(vecper_frac_digits(1/861))
OEIS_recurrence_guess(concat(vector(20,i,v)))
quit

vector(20,n, vecper_periodic_length(vecper_frac_digits(1/(11*n))))
vector(20,n, vecper_periodic(vecper_frac_digits(1/(11*n))))

a(n,mult=11) =
{
  if(n==0,return(0));
  my(f=1/(mult*n));
  if(f==1,return(0));
  while(f<1/10, f*=10);
  my(v=vecper_periodic(vecper_frac_digits(f)));
  fromdigits(v)
}
\\ a(1,1)
\\ vecper_periodic(vecper_frac_digits(1))

\\ A036275 all n
my(v=OEIS_data("A036275")); vector(#v,n, a(n,1)) == v || error();

\\ OEIS_samples("A036275")
\\ vector(36,n, a(2*n+1,1))
\\ quit

want={[    0, 90, 45, 30, 27, 18, 15, 129870, 36, 10, 90,
              8264462809917355371900, 75, 699300, 649350, 60, 81,
              5347593582887700, 50, 479468899521531100, 45, 432900,
              4132231404958677685950, 3952569169960474308300, 78]};
vector(#want,n,n--; a(n))
vector(#want,n,n--; a(n)) - want
print("want "want[19 +1])
print("a    "a(19))

\\ 478
quit

default(realprecision,50)
1/(19*11.0)

\\-----------------------------------------------------------------------------
\\ A337212
\\
\\ n-bonacci sequence mod 3 is eventually periodic
\\
\\ 1, 8, 13, 26, 104, 728, 364, 80, 91, 8744, 3851, 3280, 59048, 4782968, 7174453

nbonacci_mod_3(n) =
{
  vecper_make_state((v)->[v[1], concat(v[2..n], vecsum(v)%3)],
                    vector(n,i,i>1));
}
{
  for(n=1,100,
     my(v=nbonacci_mod_3(n),
        vv=vecper_normalize(v));
     print(apply(length,v)," ",apply(length,v));
     if(v!=vv,error()));
}
quit


\\-----------------------------------------------------------------------------

{
  for(n=2,20,
     my(x,y,s); [x,y,s]=vecper_contfrac_to_xys([[],vector(n,i,if(i==n,2,1))]);
     print1(s,","));
  quit;
}

vecper_contfrac_of_xys(0,1,s)


\\-----------------------------------------------------------------------------

\\ f=.333333333
f=0.0
f==2*f
f===2*f
print("num ",numerator(f));
print("den ",denominator(f));
print("divrem ",divrem(2*f,1));
binary(f)
{
  for(i=1,10,
     my(qr=divrem(2*f,1));
     f=qr[2];
     print(" digit "qr[1]" to rem "f));
}

\\ vecper_frac_digits(f)

quit

\\-----------------------------------------------------------------------------
