\\ Copyright 2019, 2020, 2022 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(recover,0);
default(strictargs,1);

read("vecper.gp");
read("vecper-wip.gp");  \\ check multiple reads ok
read("vecper-wip.gp");
read("vecper-wip.gp");
read("test-helpers.gp");

option_verbose = 0;
check_count = 0;

\\ informational diagnostics
print("default(secure) is "default(secure));
print("have_lex_of_quad  = "have_lex_of_quad);
print("have_test_is_gnil = "have_test_is_gnil);
print("test_is_gnil  ",test_is_gnil);



\\-----------------------------------------------------------------------------
\\ vecper_contfrac_of_xys()

{
  check_equal(vecper_contfrac_of_xys(1,0,0), [[1],[]],
              "vecper_contfrac_of_xys() 1 + 0*sqrt(0)");

  check_equal(vecper_contfrac_of_xys(-2,1,4), [[0],[]],
              "vecper_contfrac_of_xys() -2 + sqrt(4)");
  check_equal(vecper_contfrac_of_xys(-2,2,1), [[0],[]],
              "vecper_contfrac_of_xys() -2 + 2*sqrt(2)");
  check_equal(vecper_contfrac_of_xys(2,-2,1), [[0],[]],
              "vecper_contfrac_of_xys() 2 - 2*sqrt(2)");
  check_equal(vecper_contfrac_of_xys(-1/2,1,1/4), [[0],[]],
              "vecper_contfrac_of_xys() -1/2 + sqrt(1/4)");
  check_equal(vecper_contfrac_of_xys(7,1,4), [[9],[]],
              "vecper_contfrac_of_xys() 7 + sqrt(4)");
}


\\-----------------------------------------------------------------------------
\\ vecper_contfrac_to_xys()

{
  check_equal(contfracpnqn([]),   [1,0;0,1]);
  check_equal(contfracpnqn([0]),   [0,1;1,0]);
  check_equal(contfracpnqn([0,0]), [1,0;0,1]);

  check_equal('c*('x+'y*'s) + 'd,
              'x*'c+'d  + 'y*'c*'s);
  check_equal(('x*'c+'d  + 'y*'c*'s)*('x*'c+'d  - 'y*'c*'s) == 
              ('x*'c+'d)^2 - ('y*'c*'s)^2, 1);

  check_equal(('x*'a+'b  + 'y*'a*'s)*('x*'c+'d  - 'y*'c*'s) == 
              ('x*'a+'b)*('x*'c+'d)
              - 'y*'a*'s   * 'y*'c*'s
              - ('x*'a+'b) * 'y*'c*'s
              + 'y*'a*'s * ('x*'c+'d), 1);

  check_equal(vecper_contfrac_to_xys([[],[1]]),
              [1/2, 1/2, 5],
              "vecper_contfrac() phi");

  \\ F = 1/2 + 1/F
  \\ F^2 = 1/2*F + 1
  \\ F^2 - 1/2*F - 1
  check_equal(vecper_contfrac_to_xys([[], [1, 1/2]]),
              [1/2, 1, 9/4],
              "vecper_contfrac() [1, 1/2, ...] = 2");
  check_equal(vecper_contfrac_to_quad([[], [1, 1/2]]),
              2,
              "vecper_contfrac_to_quad() [1, 1/2, ...] = 2");
}

\\-----------------------------------------------------------------------------
\\ vecper_contfrac()

{
  \\ 4.236067
  \\ 1 / .236067 = 4.236067 again
  check_equal(vecper_contfrac(quadgen(20)+2), [[],[4]]);
  check_equal(vecper_contfrac_to_xys([[],[4]]),
              [2, 1/2, 20],
              "vecper_contfrac() sqrt5+2");

  check_equal(vecper_contfrac(7 + (1+quadgen(20))/2), [[8],[1]]);
  check_equal(vecper_contfrac_to_xys([[8],[1]]),
              [7+1/2, 1/2, 5],
              "vecper_contfrac() phi+7");

  check_equal(vecper_contfrac(-13 + (1+quadgen(20))/2), [[-12],[1]]);
  check_equal(vecper_contfrac_to_xys([[-12],[1]]),
              [-13+1/2, 1/2, 5],
              "vecper_contfrac() phi+7");
}

check_equal(vecper_contfrac(0), [[0],[]]);
check_equal(vecper_contfrac(1/3), [[0,3],[]]);
check_equal(contfrac(4/17), [0,4,4]);
check_equal(vecper_contfrac(4/17), [[0,4,4],[]]);
check_equal(contfracpnqn([]), matid(2));
check_equal([]==0, 1);
check_equal([0,0]==0, 1);
check_equal(vecper_contfrac_to_xys([[0,3],[]]), [1/3,0,0], \
            "vecper_contfrac_to_xys() 0,3");
check_equal(denominator([1/2,1/3]), 6);
{
  my(v=[0,5,'x],
     m=contfracpnqn(v),
     f=m[1,1]/m[2,1]);
  check_equal(m, ['x,1; 5*'x+1, 5]);
  check_equal(f, 0 + 1/(5 + 1/'x));
}
{
  my(v=[7,5,13],
     m=contfracpnqn(v),
     f=m[1,1]/m[2,1]);
  check_equal(f, 7+1/(5+1/13));
  check_equal(m, [475, 36;
                   66,  5]);
  check_equal(7+1/(5+1/13), 475/66);

  v=concat(v,['x]);
  m=contfracpnqn(v);
  check_equal(m, [475*'x + 36, 475;
                   66*'x +  5, 66]);
}
{
  check_equal(contfracpnqn([17,19]), [324, 17;
                                       19,  1]);
  check_equal(contfracpnqn([17,19,'x]), [324*'x + 17, 324;
                                          19*'x + 1,   19]);
  \\ x = (324*'x + 17)/(19*'x + 1)
  \\ x*(19*'x + 1) = 324*'x + 17
  \\ polroots(19*x^2 + x - 324*x - 17)
  \\ polroots(19*x^2 - 323*x - 17)
  \\ 17/2 + 1/38*sqrt(105621)
  check_equal(vecper_contfrac_to_xys([[],[17,19]]),
              [17/2, 1/38, 105621],
              "vecper_contfrac_to_xys() 17,19");

  \\ my(m=contfracpnqn(concat(vector(20,i,[17,19])))); m[1,1]/m[2,1]*1.0
}
{
  my(sqrt2=quadgen(8));
  check_equal(sqrt2^2 == 2, 1, "sqrt2^2");
  check_equal(sqrt2.disc, 8);
  check_equal(1/(sqrt2-1), 1 + sqrt2);
  check_equal(vecper_INTERNAL_floor_of_quad(sqrt2), 1);
  check_equal(vecper_contfrac_of_xys(0,1,2), [[1],[2]],
              "vecper_contfrac_of_xys() sqrt2");
  check_equal(vecper_contfrac(sqrt2), [[1],[2]],
              "vecper_contfrac() sqrt2");
  check_equal(contfracpnqn([2]), [2, 1;
                                  1, 0]);
  check_equal(contfracpnqn([2,'x]), [2*'x+1, 2;
                                     'x,     1]);
  check_equal(nearly_equal(0 + 1/2*sqrt(8),
                           sqrt(2)),  1);
  check_equal(vecper_contfrac_to_xys([[1],[2]]),
              [0, 1/2, 8],
              "vecper_contfrac_to_xys() sqrt2");
  check_equal(vecper_contfrac_to_quad([[1],[2]]), sqrt2,
              "vecper_contfrac_to_quad() sqrt2");
}
{
  my(sqrt5=quadgen(20));
  check_equal(sqrt5^2 == 5, 1, "sqrt5^2");
  check_equal(sqrt5.disc, 20);
  check_equal(vecper_INTERNAL_floor_of_quad(sqrt5), 2);
  check_equal(vecper_contfrac(sqrt5), [[2],[4]]);
  check_equal(nearly_equal(0 + 1/2*sqrt(20),
                           sqrt(5)),  1);
  check_equal(vecper_contfrac_to_xys([[2],[4]]), [0,1/2,20],
              "vecper_contfrac_to_xys() sqrt5");
  check_equal(vecper_contfrac_to_quad([[2],[4]]), sqrt5,
              "vecper_contfrac_to_quad() sqrt5");

  check_equal(vecper_contfrac(-2 + sqrt5), [[0],[4]],
             "vecper_contfrac() -2 + sqrt5");
  check_equal(vecper_contfrac( 2 + sqrt5), [[],[4]]);
  check_equal(vecper_contfrac_to_quad([[],[4]]), 2 + sqrt5);

  my(phi=(1+sqrt5)/2);
  check_equal(vecper_contfrac(phi), [[],[1]]);
  check_equal(vecper_contfrac_to_quad([[],[1]]), phi);
  check_equal(vecper_contfrac_to_xys([[],[1]]), [1/2, 1/2, 5]);
  check_equal(1/2 + 1/2*sqrt5, phi);
}

\\vecper_contfrac_to_xys([[2],[5,7,2]])
{
  if(0,
     for(n=1,50000,
        my(v=vecper_contfrac_of_xys(0,1,n));
        #v[2]==3 || next;
     \\ #vecmin(v[2])>=2 || next;
        v[1][1] != v[2][1] || next;
        print(n" "v"  "core(n,1));
     );
     quit;
    );
}
{
  check_equal(vecper_contfrac_of_xys(0,1,4),  [[2], []],
              "vecper_contfrac_of_xys() 4");

  check_equal(vecper_contfrac_of_xys(0,1,8),  [[2], [1,4]],
              "vecper_contfrac_of_xys() 8");
  check_equal(vecper_contfrac_to_xys([[2], [1,4]]), [0,1/2,32]);
  check_equal(32*(1/2)^2, 8);
}

{
  \\ 1/2 + 1/2*sqrt(5)      odd discriminant 4d+1
  my(phi=quadgen(5));
  check_equal((2*(phi-1/2))^2 == 5, 1, "phi squared up");
  check_equal(phi.disc, 5);
  check_equal(nearly_equal(phi, (1+sqrt(5))/2), 1);
  check_equal(vecper_contfrac(phi), [[],[1]]);
}
{
  \\ 1/2 + 1/2*sqrt(29)      odd discriminant 4d+1
  my(q=quadgen(29));
  check_equal((2*q-1)^2 == 29, 1, "q squared up");
  check_equal(q.disc, 29);
  check_equal(nearly_equal(q, (1+sqrt(29))/2), 1);
  check_equal(vecper_contfrac(q), [[3],[5]]);
  \\ 3 + 1/(5 + 1/(5 + 1/(...)))
  \\ p = 1/(5 + p)
  \\ p*(p+5) = 1
  \\ p^2 + 5*p - 1 = 0
  check_equal(poldisc(x^2 + 5*x - 1), 29);
  check_equal(subst('x^2 + 5*'x - 1, 'x,q-3) == 0, 1);  
  my(m=contfracpnqn([3,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5]));
  check_equal(nearly_equal(m[1,1]/m[2,1], q), 1);
}

{
  \\ perfect square s
  check_equal(vecper_contfrac_of_xys(0,1,9), [[3],[]],
              "vecper_contfrac_of_xys() 0 + 1*9");
}


\\-----------------------------------------------------------------------------
\\ vecper_ensure_fixed()

check_equal(vecper_ensure_fixed([[],[]],0), [[],[]]);
check_equal(vecper_ensure_fixed([[10,20,30],[40]],0), \
            [[10,20,30],[40]]);
check_equal(vecper_ensure_fixed([[10,20,30],[40]],3), \
            [[10,20,30],[40]], \
            "vecper_ensure_fixed() 3");
check_equal(vecper_ensure_fixed([[10,20,30],[40]],4), \
            [[10,20,30,40],[40]], \
            "vecper_ensure_fixed() 4");
check_equal(vecper_ensure_fixed([[],[1,2]],1), [[1],[2,1]]);
check_equal(vecper_ensure_fixed([[],[1,2]],2), [[1,2],[1,2]]);
check_equal(vecper_ensure_fixed([[],[1,2]],3), [[1,2,1],[2,1]]);

{
  for(fixed=0,4,
     for(periodic=1,5,
        my(v=[vector(fixed,i,-10*i), vector(periodic,i,10*i)]);
        for(len=0, fixed+3*periodic+5,
           my(new_v=vecper_ensure_fixed(v,len));
           check_equal(vecper_is_equal(v,new_v), 1,
                       "vecper_ensure_fixed()"))));
}              


\\-----------------------------------------------------------------------------
\\ vecper_set()

\\ NOT WORKING
\\
\\ check_equal(vecper_set([[11,12,13,14],[]],3,x), \
\\             [[11,12,x,14],[]], \
\\             "vecper_set()");
\\ check_equal(vecper_set([[11,12,13,14],[0]],7,x), \
\\             [[11,12,13,14,0,0,x],[0]]);
\\ check_equal(vecper_set([[11,12,13,14], [21,22]], 7, x), \
\\             [[11,12,13,14,21,22,x,22], [21,22]]);
\\ check_equal(vecper_set([[11,12,13,14], [21,22]], 5, x, 2), \
\\             [[11,12,13,14], [x,22]]);
\\ check_equal(vecper_set([[11,12,13,14], [21,22]], 5, x, 4), \
\\             [[11,12,13,14], [x,22,21,22]]);

\\ {
\\   for(fixed=0,4,
\\      for(periodic=1,5,
\\         my(vecper=[vector(fixed,i,-10*i), vector(periodic,i,10*i)]);
\\         for(pos=1, fixed+3*periodic+5,
\\            for(setperiod=0, 4*periodic,
\\               print(vecper" pos="pos" setperiod="setperiod);
\\               my(new_vecper=vecper_set(vecper,pos,'x,setperiod),
\\                  len = (setperiod+1)*(periodic+1)*3 + fixed + 20,
\\                  new_vec = vecper_get_range(new_vecper,1,len),
\\                  old_vec = vecper_get_range(vecper,1,len));
\\ 
\\               old_vec[pos]='x;
\\               if(setperiod>0,
\\                  forstep(i=pos,#old_vec,setperiod,
\\                     old_vec[i]='x));
\\ 
\\               check_equal(new_vec,old_vec)))));
\\ }
              
     


\\-----------------------------------------------------------------------------
\\ vecper_concat_vecs()

check_equal(vecper_concat_vecs([[],[]]), [[],[]], "vecper_concat_vecs() empty");
check_equal(vecper_concat_vecs([[ [2,1] ],[]]), [[2,1],[]]);
check_equal(vecper_concat_vecs([[],[ [2,1] ]]), [[],[2,1]]);
check_equal(vecper_concat_vecs([[],[ [2,1],[4,3] ]]), [[],[2,1,4,3]]);


\\-----------------------------------------------------------------------------
\\ vecper_make_state()

{
  check_equal(vecper_make_state((state)->[10*state,(state+1)%5]),
              [[], [0,10,20,30,40]]);

  check_equal(vecper_make_state((state)->[10*state,(state+1)%5], -1),
              [[-10], [0,10,20,30,40]]);

  check_equal(vecper_make_state((state)->divrem(10*state,7), 3),
              [[], [4,2,8,5,7,1]]);
  check_equal(vecper_frac_digits(3/7), [[], [4,2,8,5,7,1]]);

  check_equal(
              vecper_make_state((state)->[10*state,(state+1)%3],
                                -1),
              [[-10], [0,10,20]]);
}


\\-----------------------------------------------------------------------------
\\ vecper_add()

\\ check_equal(vecper_add([[5],[]], [[7],[]]), [[12],[]]);
\\ check_equal(vecper_add([[5],[]], [[7,9],[]]), [[12,9],[]]);
\\ check_equal(vecper_add([[5,9],[]], [[7],[]]), [[12,9],[]]);


\\-----------------------------------------------------------------------------
\\ vecper_INTERNAL_vector_rotate_down()

check_equal(vecper_INTERNAL_vector_rotate_down([]), [], \
            "vecper_INTERNAL_vector_rotate_down() empty");
check_equal(vecper_INTERNAL_vector_rotate_down([1]), [1]);
check_equal(vecper_INTERNAL_vector_rotate_down([10,20]), [20,10]);
check_equal(vecper_INTERNAL_vector_rotate_down([10,20,30]), [20,30,10]);


\\-----------------------------------------------------------------------------
\\ vecper_unroll()

check_equal(vecper_unroll([ []]), [0, []]);
check_equal(vecper_unroll([ []]), [0, []]);
check_equal(vecper_unroll([ [1]]), [1, [1]]);
check_equal(vecper_unroll([ [1,2,3]]), [1, [2,3,1]]);
check_equal(vecper_unroll([1, []]), [1,0, []]);
check_equal(vecper_unroll([1, [9]]), [1,9, [9]]);
check_equal(vecper_unroll([1,2, [7,8,9]]), [1,2,7, [8,9,7]]);


\\-----------------------------------------------------------------------------
\\ vecper_make()
check_equal(vecper_make([4,5],[6,7,8]), [[4,5],[6,7,8]])


\\-----------------------------------------------------------------------------
